-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-- Used to build a mask for summoning pets
-- NOTE: The names must match the values returned by 'GetInstanceInfo()'
local petMasks =
{
  ["Kalimdor"]         = 0x0001,
  ["Eastern Kingdoms"] = 0x0002,
  ["Outland"]          = 0x0004,
  ["Northrend"]        = 0x0008,
  ["city"]             = 0x0010,
  ["party"]            = 0x0020,
  ["raid"]             = 0x0040,
  ["arena"]            = 0x0080,
  ["pvp"]              = 0x0100,
  ["Pandaria"]         = 0x0200
};

local mountSpells =
{
  -- Aquatic Form
  {
    ["id"]    = 1066,
    ["flags"] = 0x08
  },
  -- Swift Flight Form
  {
    ["id"]    = 40120,
    ["flags"] = 0x02,
    ["addButton"] =
      function()
        return ((IsSpellKnown(40120)) or (not IsSpellKnown(33943)));
      end
  },
  -- Flight Form
  {
    ["id"]    = 33943,
    ["flags"] = 0x02,
    ["addButton"] =
      function()
        return IsSpellKnown(33943);
      end
  },
  -- Travel Form
  {
    ["id"]    = 783,
    ["flags"] = 0x01
  },
  -- Ghost Wolf
  {
    ["id"]    = 2645,
    ["flags"] = 0x01
  },
  -- Running Wild
  {
    ["id"]    = 87840,
    ["flags"] = 0x01
  },
  -- Aspect of the Cheetah
  {
    ["id"]    = 5118,
    ["flags"] = 0x01,
  },
  -- Herbouflage
  {
    ["id"]    = 88715,
    ["flags"] = 0x00
  },
};

local mountPriority =
{
  [ 1066] = 3, -- Aquatic Form
  [40120] = 3, -- Swift Flight Form
  [33943] = 3, -- Flight Form
  [  783] = 1, -- Travel Form
  [ 2645] = 1, -- Ghost Wolf
  [ 5118] = 1, -- Aspect of the Cheetah
};

local mountHolidays =
{
  [NigglesHolidayHallowsEnd] =
  {
    [37011] = {["skill"] = 1, ["flags"] = 0x03}, -- Magic Broom
    [33176] = {["skill"] = 4, ["flags"] = 0x03}  -- Flying Broom
  },
};

local mountRestricted =
{
  [75207] = -- Abyssal Seahorse
  {
    [L["Shimmering Expanse"]] = 1,
    [L["Abyssal Depths"]]     = 1,
    [L["Kelp'thar Forest"]]   = 1
  },
  [25953] = -- Blue Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26056] = -- Green Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26054] = -- Red Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  },
  [26055] = -- Yellow Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = 1
  }
};

local zoneRestricted = {};

local mountIdealFlags =
{
  [75614] = 0x03, -- Celestial Steed
  [ 2645] = 0x00, -- Ghost Wolf
  [48025] = 0x0B, -- Headless Horseman's Mount
  [72286] = 0x03, -- Invincible
  [64731] = 0x08, -- Sea Turtle
  [  783] = 0x00, -- Travel Form
  [ 5118] = 0x00, -- Aspect of the Cheetah
};

local ridingInfo =
{
  33388, -- Apprentice Riding
  33391, -- Journeyman Riding
  34090, -- Artisan Riding
  34091, -- Expert Riding
  90265  -- Master Riding
};

local mountEnv =
{
  {label = "Ground", action = "Run",  flag = 0x01},
  {label = "Air",    action = "Fly",  flag = 0x02},
  {label = "Water",  action = "Swim", flag = 0x08}
};

local petFlagSettings =
{
  [LE_PET_JOURNAL_FLAG_COLLECTED]     = true,
  [LE_PET_JOURNAL_FLAG_FAVORITES]     = false,
  [LE_PET_JOURNAL_FLAG_NOT_COLLECTED] = false
};

-- Pets used as part of a quest
local petQuest =
{
  [31880] = -- Blood Elf Orphan Whistle
  {
    "all"
  },
  [31881] = -- Draenei Orphan Whistle
  {
    "all"
  },
  [30803] = -- Felhound Whistle
  {
    L["Hellfire Peninsula"]
  },
  [36865] = -- Golem Control Unit
  {
    L["Dun Argol"]
  },
  [36865] = -- Golem Control Unit
  {
    "all"
  },
  [18598] = -- Human Orphan Whistle
  {
    "all"
  },
  [46397] = -- Oracle Orphan Whistle
  {
    "all"
  },
  [18597] = -- Orcish Orphan Whistle
  {
    "all"
  },
  [34253] = -- Sizzling Ember
  {
    L["Throne of Kil'jaeden"]
  },
  [46362] =
  {
    L["Silithus"],
    L["Tanaris"],
    L["Un'Goro Crater"]
  },
  [68646] = -- Winterspring Cub
  {
    L["Winterspring"]
  },
  [46396] = -- Wolvar Orphan Whistle
  {
    "all"
  },
  [31815] = -- Zeppit's Crystal
  {
    L["Netherstorm"]
  }
};

local petHoliday =
{
  [15698] = NigglesHolidayWinterVeil, -- Father Winter's Helper
  [15705] = NigglesHolidayWinterVeil  -- Winter's Little Helper
};

local restrictedZones = {};
local continentalFlight = {};
local mountMacroText = "";
local ridingSkill = 0;
local hasClassRaceMounts = false;
local filteredMounts = {};
local mountJournalToggles = {};
local mountJournalFilters =
{
  environment  = 0x0B,
  onlySelected = false,
  onlyUsable   = false
};

StaticPopupDialogs["NIGGLES_COMPANIONS_CLEARALL"] =
{
  text         = "%s",
  button1      = OKAY,
  button2      = CANCEL,
  whileDead    = 1,
  OnAccept     = function(self, data)
    NigglesOptionsCompanionsClearAll(data);
  end,
  showAlert    = 1,
  timeout      = 0,
  exclusive    = 1,
  hideOnEscape = 1
};
StaticPopupDialogs["NIGGLES_COMPANIONS_SELECTALL"] =
{
  text         = "%s",
  button1      = OKAY,
  button2      = CANCEL,
  whileDead    = 1,
  OnAccept     = function(self, data)
    NigglesOptionsCompanionsSelectAll(data);
  end,
  showAlert    = 1,
  timeout      = 0,
  exclusive    = 1,
  hideOnEscape = 1
};

--  Copied from 'Blizzard_PetJournal.lua'
local PLAYER_MOUNT_LEVEL  = 20;
local MOUNT_BUTTON_HEIGHT = 46;

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to initialise the Niggles Mount and Pet functionality
--
function NigglesCompanionsInit()
  -- Check the Mount Journal scroll frame exists
  if (MountJournal.ListScrollFrame ~= nil) then
    -- Hook the function that updates the Mounts scroll frame
    hooksecurefunc("MountJournal_UpdateMountList",
      function ()
        NigglesMountsListUpdate(MountJournal.ListScrollFrame);
      end);
    MountJournal.ListScrollFrame.update = MountJournal_UpdateMountList;

    -- Create the Niggles Mounts Journal and toggles, if required
    NigglesMountJournalEnable();
  end

  -- Check the Pets Journal scroll frame exists
  if (PetJournal.listScroll ~= nil) then
    -- Hook the function that updates the Pets scroll frame
    hooksecurefunc("PetJournal_UpdatePetList",
      NigglesPetsListUpdate);
    PetJournal.listScroll.update = PetJournal_UpdatePetList;

    -- Create the Pets Journal toggles, if required
    NigglesPetTogglesEnable();
  end

  return;
end

--
-- Function to enable/disable the Niggles Mount Journal
--
function NigglesMountJournalEnable()
  -- Check the Mount Journal exists
  if (MountJournal ~= nil) then
    -- Local Variables
    local frame;
    local maxButtons = 0;
    local numButtons = 0;
    local numEnvironments = #mountEnv;
    local texture;
    local toggle;

    -- Create the Niggles Mount Journal, if required
    if ((NigglesDBC.settings.UiMountsFilter == true) and
        (NigglesMountJournal == nil)) then
      local frame = CreateFrame("FRAME", "NigglesMountJournal", MountJournal,
        "NigglesMountJournalTemplate");
      frame.listScroll.update = NigglesMountJournalUpdate;
      frame.listScroll.scrollBar.doNotHide = true;
      HybridScrollFrame_CreateButtons(frame.listScroll,
        "MountListButtonTemplate", 44, 0);
    end

    -- Work out how many sets of toggles are required
    if (NigglesDBC.settings.MountsEnabled == true) then
      if (NigglesDBC.UiMountsFilter == true) then
        maxButtons = math.max(#(NigglesMountJournal.listScroll.buttons),
          #(MountJournal.ListScrollFrame.buttons));
      else
        maxButtons = #(MountJournal.ListScrollFrame.buttons);
      end
    end

    -- Create toggles for each button
    for buttonIdx = 1, maxButtons do
      -- Initialise the array, if required
      if (mountJournalToggles[buttonIdx] == nil) then
        mountJournalToggles[buttonIdx] = {};
      end

      -- Create a toggle for each environment for the button
      for toggleIdx = 1, numEnvironments do
        if (mountJournalToggles[buttonIdx][toggleIdx] == nil) then
          -- Create the toggle 
          toggle = CreateFrame("CheckButton",
            "NigglesMountToggle"..buttonIdx..mountEnv[toggleIdx].action,
            NigglesMountJournal, "NigglesCompanionToggleTemplate");
          toggle:SetID(toggleIdx);
          toggle:SetScript("OnEnter", NigglesMountToggleOnEnter);
          toggle:SetScript("OnClick", NigglesMountToggleOnClick);
           
          -- Adjust the appearance of the disabled texture
          texture = toggle:GetDisabledTexture();
          texture:SetDesaturated(1);
          texture:SetAlpha(.5)

          -- Save the toggle
          mountJournalToggles[buttonIdx][toggleIdx] = toggle;
        end
      end
    end

    -- Show/Hide the toggles
    if (NigglesDBC.settings.MountsEnabled == true) then
      -- Work out which buttons to assign the toggles to
      if (NigglesDBC.settings.UiMountsFilter == true) then
        buttons = NigglesMountJournal.listScroll.buttons;
      else
        buttons = MountJournal.ListScrollFrame.buttons;
      end
      numButtons = #buttons;

      -- Show the toggles for the buttons
      for buttonIdx = 1, numButtons do
        for toggleIdx = 1, numEnvironments do
          toggle = mountJournalToggles[buttonIdx][toggleIdx];
          toggle:SetParent(buttons[buttonIdx]);
          toggle:SetPoint("BOTTOMRIGHT", buttons[buttonIdx], "BOTTOMRIGHT",
            -(5+((numEnvironments-toggleIdx)*16)), 5);
          toggle:Show();
        end
      end
    else
      numButtons = #mountJournalToggles;
      for buttonIdx = 1, numButtons do
        for toggleIdx = 1, numEnvironments do
          if (mountJournalToggles[buttonIdx][toggleIdx] ~= nil) then
            mountJournalToggles[buttonIdx][toggleIdx]:Hide();
          end
        end
      end
    end
     
    -- Show the appropriate frame
    if (NigglesDBC.settings.UiMountsFilter == true) then
      MountJournal.ListScrollFrame:Hide();
      NigglesMountJournal:Show();
      NigglesMountJournalFilter();
    else
      if (NigglesMountJournal ~= nil) then
        NigglesMountJournal:Hide();
      end
      MountJournal.ListScrollFrame:Show();
    end
  end

  return;
end

--
-- Function to enable/disable the toggles for the Pets Journal
--
function NigglesPetTogglesEnable()
  -- Local Variables
  local button;
  local numButtons;
  local scrollFrame;
  local texture;
  local toggle;

  -- Check the Pet Journal scroll frame exists
  if ((PetJournal ~= nil) and
      (PetJournal.listScroll ~= nil)) then
    -- Initialise some variables
    scrollFrame = PetJournal.listScroll;
    numButtons  = #scrollFrame.buttons;

    -- Create, show or hide the toggle for each button
    for buttonIdx = 1, numButtons do
      -- Get the current button
      button = scrollFrame.buttons[buttonIdx];

      -- Check if the Pets option is enabled
      if (NigglesDBC.settings.PetsEnabled == true) then
        -- Check if the toggle needs to be created
        if (button.nigglesToggle == nil) then
          -- Create the toggle and position it
          toggle = CreateFrame("CheckButton", "NigglesPetToggle"..buttonIdx,
            button, "NigglesCompanionToggleTemplate");
          toggle:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -5, 5);
          toggle:SetScript("OnEnter", NigglesPetToggleOnEnter);
          toggle:SetScript("OnClick", NigglesPetToggleOnClick);
          button.nigglesToggle = toggle;

          -- Adjust the appearance of the disabled texture
          texture = toggle:GetDisabledTexture();
          texture:SetDesaturated(1);
          texture:SetAlpha(.5)
        end

        -- Show the toggle
        button.nigglesToggle:Show();
      else
        -- Hide the toggle
        if (button.nigglesToggle ~= nil) then
          button.nigglesToggle:Hide();
        end
      end
    end

    -- Update the Pets scroll frame, if required
    if (PetJournal.listScroll:IsShown()) then
      NigglesPetsListUpdate();
    end
  end

  return;
end

--
-- Function to filter the Niggles Mount Journal
--
function NigglesMountJournalFilter()
  -- Local Variables
  local numMounts = GetNumCompanions("MOUNT");
  local spellId;
  local name;
  local filterName;

  -- Get the filter values
  filterName = NigglesMountJournalSearchBox:GetText();
  if (filterName == SEARCH) then
    filterName = nil;
  else
    filterName = filterName:lower();
  end
  
  -- Filter the mounts
  wipe(filteredMounts);
  for idx = 1, numMounts do
    -- Get the required info for the mount
    _, name, spellId, _, _, mountFlags = GetCompanionInfo("MOUNT", idx);

    -- Check if the mount matches the filter
    if ((name ~= nil) and 
        ((filterName == nil) or
         (name:lower():find(filterName, 1, true) ~= nil)) and
        (bit.band(mountFlags, mountJournalFilters.environment) ~= 0) and
        ((not mountJournalFilters.onlySelected) or
         (NigglesDBC.mounts[spellId] ~= nil)) and
        ((not mountJournalFilters.onlyUsable) or
         (IsUsableSpell(spellId)))) then
      table.insert(filteredMounts, idx);
    end
  end

  -- Update the Niggles Mount Journal
  NigglesMountJournalUpdate();

  return;
end

--
-- Function to update the Niggles Mount Journal List
--
-- NOTE: This is a modified version of 'MountJournal_UpdateMountList()' in
--       'Blizzard_PetJournal.lua'.
--
--
function NigglesMountJournalUpdate()
  -- Modified for Niggles
  local scrollFrame = NigglesMountJournal.listScroll;
  local offset = HybridScrollFrame_GetOffset(scrollFrame);
  local buttons = scrollFrame.buttons;

  -- Modified for Niggles
  local numMounts = (#filteredMounts or 0);

  local showMounts = 1;
  local playerLevel = UnitLevel("player");
  -- Modified for for Niggles 
  -- if  ( numMounts < 1 ) then
  --   -- display the no mounts message on the right hand side
  --   MountJournal.MountDisplay.NoMounts:Show();
  --   showMounts = 0;
  -- else
  --   MountJournal.MountDisplay.NoMounts:Hide();
  -- end

  for i=1, #buttons do
    local button = buttons[i];

    -- Modified for Niggles
    if (offset+i <= numMounts) then
      index = filteredMounts[offset+i];
    else
      index = -1;
    end

    -- Modified for Niggles
    if ((index > 0) and (showMounts == 1)) then
      local creatureID, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", index);
      button.name:SetText(creatureName);
      button.icon:SetTexture(icon);
      button.index = index;
      button.spellID = spellID;

      button.active = active;
      if ( active and playerLevel >= PLAYER_MOUNT_LEVEL) then
        button.DragButton.ActiveTexture:Show();
      else
        button.DragButton.ActiveTexture:Hide();
      end
      button:Show();
      
      if ( MountJournal.selectedSpellID == spellID ) then
        button.selected = true;
        button.selectedTexture:Show();
      else
        button.selected = false;
        button.selectedTexture:Hide();
      end
      button:SetEnabled(1);
      if (playerLevel >= PLAYER_MOUNT_LEVEL) then
        button.DragButton:SetEnabled(1);
        button.additionalText = nil;
        button.icon:SetDesaturated(0);
        button.icon:SetAlpha(1.0);
        button.name:SetFontObject("GameFontNormal");        
      else
        button.DragButton:SetEnabled(0);
        button.additionalText = MOUNT_JOURNAL_CANT_USE;
        button.icon:SetDesaturated(1);
        button.icon:SetAlpha(.5);
        button.name:SetFontObject("GameFontDisable");        
      end

      if ( button.showingTooltip ) then
        MountJournalMountButton_UpdateTooltip(button);
      end
    elseif (index <= 0) then
      button:Hide();
    else
      button.icon:SetTexture("Interface\\PetBattles\\MountJournalEmptyIcon");
      button.index = index;
      button.spellID = 0;
      button.selected = false;
      button.DragButton.ActiveTexture:Hide();
      button.selectedTexture:Hide();
      button:SetEnabled(0);
      button.DragButton:SetEnabled(0);
      button.icon:SetDesaturated(1);
      button.icon:SetAlpha(0.5);
    end
  end

  local totalHeight = numMounts * MOUNT_BUTTON_HEIGHT;
  HybridScrollFrame_Update(scrollFrame, totalHeight, scrollFrame:GetHeight());
  -- Modified for Niggles
  MountJournal.MountCount.Count:SetText(GetNumCompanions("MOUNT"));
  if ( showMounts == 0 ) then
    MountJournal.selectedSpellID = 0;
    MountJournal_UpdateMountDisplay();
    MountJournal.MountCount.Count:SetText(0);
  end
  if ( playerLevel >= PLAYER_MOUNT_LEVEL and numMounts > 0) then
    MountJournal.MountButton:SetEnabled(1);
  else
    MountJournal.MountButton:SetEnabled(0);
  end

  -- Modified for Niggles
  NigglesMountsListUpdate(scrollFrame);
end

--
-- Function called when the contents of the player's bags changes. It searches
-- for any quest items used to summon pets
--
function NigglesPetsBagUpdate()
  -- Clear the list of restricted zones
  wipe(restrictedZones);

  -- Search for any items used to summon quest pets
  for itemId, itemZones in pairs(petQuest) do
    if (GetItemCount(itemId, false, false) > 0) then
      if (itemZones[1] == "all") then
        restrictedZones["all"] = true;
        break;
      else
        for _, zoneName in ipairs(itemZones) do
          restrictedZones[zoneName] = true;
        end
      end
    end
  end

  return;
end

--
-- Function to set the mask used to determine if pets should be summoned
--
function NigglesPetsSetMask()
  -- Reset the mask
  Niggles.petMask = 0;

  -- Add flags for the normal zones
  if (NigglesDBC.settings.PetsInEKingdoms == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["Eastern Kingdoms"]);
  end
  if (NigglesDBC.settings.PetsInKalimdor == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["Kalimdor"]);
  end
  if (NigglesDBC.settings.PetsInNorthrend == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["Northrend"]);
  end
  if (NigglesDBC.settings.PetsInOutland == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["Outland"]);
  end
  if (NigglesDBC.settings.PetsInPandaria == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["Pandaria"]);
  end

  -- Add flags for special types of zone
  if (NigglesDBC.settings.PetsInCities == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["city"]);
  end
  if (NigglesDBC.settings.PetsInDungeons == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["party"]);
  end
  if (NigglesDBC.settings.PetsInRaids == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["raid"]);
  end
  if (NigglesDBC.settings.PetsInArena == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["arena"]);
  end
  if (NigglesDBC.settings.PetsInBG == true) then
    Niggles.petMask = bit.bor(Niggles.petMask, petMasks["pvp"]);
  end

  return;
end

--
-- Function to register/unregister for events monitored to aid in summoning pets
--
function NigglesPetsRegisterEvents()
  -- Check if any pets have been selected for summoning
  if (next(NigglesDBC.pets) ~= nil) then
    -- Search for quest pets
    NigglesPetsBagUpdate();

    -- Register for the required events
    Niggles:RegisterEvent("BAG_UPDATE");
    Niggles:RegisterEvent("UPDATE_STEALTH");
  else
    Niggles:UnregisterEvent("BAG_UPDATE");
    Niggles:UnregisterEvent("UPDATE_STEALTH");
  end

  -- Create/destroy a timer for summoning pets
  if ((NigglesDBC.settings.PetsEnabled) and
      (next(NigglesDBC.pets) ~= nil)) then
    NigglesTimerCreate("Pets", NigglesPetSummon,
      nil, nil, 10, true);
  else
    NigglesTimerDestroy("Pets");
  end

  return;
end

--
-- Function to summon a random pet, if appropriate
--
function NigglesPetSummon()
  -- Local Variables
  local activeHolidays;
  local creatureId;
  local randomList = {{}, {}};
  local summonPet = true;
  local zoneContinent = "";
  local zoneMask = 0;
  local zoneName;
  local zoneType;

  -- Check if there is a pet already summoned
  if (C_PetJournal.GetSummonedPetID() ~= nil) then
    return;
  end

  -- Get info for the zone the player is currently in
  zoneName, zoneType = GetInstanceInfo();
  if (zoneType == "none") then
    zoneContinent = zoneName;
    zoneName      = GetRealZoneText();
    if (NigglesIsCity(zoneName) == true) then
      zoneMask = petMasks["city"];
    elseif (petMasks[zoneContinent] ~= nil) then
      zoneMask = petMasks[zoneContinent];
    else
      summonPet = false;
    end
  else
    if (petMasks[zoneType] ~= nil) then
      zoneMask = petMasks[zoneType];
    else
      summonPet = false;
    end
  end

  -- Check if any special pets might be summoned in the current area
  if ((summonPet == true) and
      (next(restrictedZones) ~= nil)) then
    if ((restrictedZones["all"] == true) or
        (restrictedZones[zoneName] == true) or
        (restrictedZones[GetSubZoneText()] == true)) then
      summonPet = false;
    end
  end

  -- Check if a pet should be summoned and can be
  if ((summonPet == true) and
      (bit.band(Niggles.petMask, zoneMask) == zoneMask) and
      (next(NigglesDBC.pets) ~= nil) and
      (IsMounted() == nil) and
      (IsFlying() == nil) and
      (IsFalling() == nil) and
      (IsStealthed() == nil) and
      (GetNumLootItems() == 0) and
      (UnitInVehicle("player") == nil) and
      (UnitUsingVehicle("player") == nil) and
      (UnitIsDeadOrGhost("player") == nil) and
      (UnitIsCharmed("player") == nil) and
      (UnitIsFeignDeath("player") == nil) and
      (UnitIsPossessed("player") == nil) and
      (UnitCastingInfo("player") == nil) and
      (UnitChannelInfo("player") == nil) and
      (UnitAura("player", L["Food"]) == nil) and
      (UnitAura("player", L["Drink"]) == nil) and
      (UnitAura("player", L["Feign Death"]) == nil) and
      (UnitAura("player", L["Camouflage"]) == nil) and
      (UnitAura("player", L["Herbouflage"]) == nil) and
      (UnitAura("player", L["Spirit of Redemption"]) == nil) and
      ((BarberShopFrame == nil) or (BarberShopFrame:IsVisible() == nil))) then

    -- Get the active holidays
    activeHolidays = NigglesHolidaysGetActive();

    -- Work out which pets have been selected by the player
    for petId, creatureId in pairs(NigglesDBC.pets) do
      if (petHoliday[creatureId] ~= nil) then
        if (bit.band(activeHolidays, petHoliday[creatureId]) ~= 0) then
          table.insert(randomList[1], petId);
        end
      else
        table.insert(randomList[2], petId);
      end
    end

    -- Call a random pet, if one isn't already summoned
    if (summonPet == true) then
      -- Check if there are any holiday pets to summon
      if (next(randomList[1]) ~= nil) then
        C_PetJournal.SummonPetByID(
          randomList[1][math.random(#randomList[1])]);
      elseif (next(randomList[2]) ~= nil) then
        C_PetJournal.SummonPetByID(
          randomList[2][math.random(#randomList[2])]);
      end
    end
  end

  return;
end

--
-- Function clear the macro of the hidden mount button after it has been
-- 'clicked'.
--
function NigglesMountPostClick(button)
  -- Clear the button's macro
  if (InCombatLockdown() == nil) then
    button:SetAttribute("macrotext", mountMacroText);
  end
end

--
-- Function to work out what to do when a macro calls '/click NigglesMount'.
-- This function is called before the button is clicked. It works out what
-- to do based on the player's current status and changes the macro of the
-- hidden button accordingly.
--
function NigglesMountPreClick(button)
  -- Local Variables
  local activeAura = "";
  local activeHolidays;
  local currentContinent;
  local currentZone;
  local isWintergraspBattle = false;
  local mapId;
  local mountFlags = nil;
  local priority;
  local randomList = {};
  local randomInfo = nil;
  local randomPriority;
  local spellName = nil;
  local wintergraspName;

  -- Get the current continent
  mapId = GetCurrentMapAreaID();
  SetMapToCurrentZone();
  currentContinent = GetCurrentMapContinent();
  SetMapByID(mapId);

  -- Get the name of the current zone
  currentZone = GetRealZoneText();

  -- Get the active holidays, if required
  if (NigglesDBC.settings.MountsHoliday == true) then
    activeHolidays = NigglesHolidaysGetActive();
  end

  -- Check if any of the mount spells are active
  for _, spellInfo in ipairs(mountSpells) do
    if (UnitAura("PLAYER", spellInfo.name) ~= nil) then
      activeAura = spellInfo.name;
    end
  end

  -- Work out what to do, based on the player's current location and state
  if (UnitInVehicle("player")) then
    -- Exit any vehicle the player is in
    VehicleExit();
  elseif (IsMounted() == 1) then
    -- Dismount from any mount the player is on
    Dismount();
  elseif ((activeAura ~= "") and
          (InCombatLockdown() == nil)) then
    -- Cancel the aura
    button:SetAttribute("macrotext", "/cancelaura "..activeAura);
  elseif (IsSwimming()) then
    if (not IsModifierKeyDown()) then
      mountFlags = {0x08, 0x02, 0x01};
    else
      mountFlags = {0x02, 0x01};
    end
  elseif (IsFlyableArea()) then
    -- Work out if the player can fly
    _, wintergraspName, isWintergraspBattle = GetWorldPVPAreaInfo(1);
    if ((not IsModifierKeyDown()) and
        (continentalFlight[currentContinent] == true) and
        ((currentContinent ~= 4) or
         (currentZone ~= wintergraspName) or
         (isWintergraspBattle ~= true))) then
      mountFlags = {0x02, 0x01};
    else
      mountFlags = {0x01};
    end
  else
    mountFlags = {0x01};
  end

  -- Check if the player should be mounted
  if (mountFlags ~= nil) then
    if (InCombatLockdown() == 1) then
      UIErrorsFrame:AddMessage(SPELL_FAILED_AFFECTING_COMBAT,
        1.0, 0.1, 0.1, 1.0);
    elseif (next(NigglesDBC.mounts) == nil) then
      UIErrorsFrame:AddMessage(L["No mounts selected"], 1.0, 0.1, 0.1, 1.0);
    else
      -- Search each specified mount type for a mount
      for _, currentFlag in ipairs(mountFlags) do
        -- Check if holiday mounts should be used
        if (NigglesDBC.settings.MountsHoliday == true) then
          -- Initialise some variables
          randomInfo = GetItemInfo;

          -- Search for selected holiday mounts with the current flag
          for holidayFlag, holidayMounts in pairs(mountHolidays) do
            -- Check if the holiday is active
            if (bit.band(activeHolidays, holidayFlag) ~= 0) then
              -- Search for usable holiday mounts
              for itemId, itemInfo in pairs(holidayMounts) do
                if ((itemInfo.skill <= ridingSkill) and
                    (bit.band(itemInfo.flags, currentFlag) ~= 0) and
                    (IsUsableItem(itemId))) then
                  table.insert(randomList, itemId);
                end
              end
            end
          end
        end

        -- Check if there are any mounts restricted to the current zone
        if ((next(randomList) == nil) and
            (zoneRestricted[currentZone] ~= nil)) then
          -- Initialise some variables
          randomInfo     = GetSpellInfo;
          randomPriority = 0;

          -- Search for selected restricted mounts with the current flag
          for _, spellId in ipairs(zoneRestricted[currentZone]) do
            if ((NigglesDBC.mounts[spellId] ~= nil) and
                (bit.band(NigglesDBC.mounts[spellId], currentFlag) ~= 0) and
                (IsUsableSpell(spellId))) then
              -- If the mount has a higher priority than the random list...
              priority = (mountPriority[spellId] or 2);
              if (priority > randomPriority) then
                -- ...reset the random list
                wipe(randomList);
                randomPriority = priority
              end

              -- Add the mount to the random list if it is high enough priority
              if (priority >= randomPriority) then
                table.insert(randomList, spellId);
              end
            end
          end
        end

        -- Search for unrestricted mounts that have the current flag
        if (next(randomList) == nil) then
          -- Initialise some variables
          randomInfo     = GetSpellInfo;
          randomPriority = 0;

          -- Search for selected mounts with the current flag
          for spellId, spellFlags in pairs(NigglesDBC.mounts) do
            if ((bit.band(spellFlags, currentFlag) ~= 0) and
                (IsUsableSpell(spellId)) and
                (mountRestricted[spellId] == nil)) then
              -- If the mount has a higher priority than the random list...
              priority = (mountPriority[spellId] or 2);
              if (priority > randomPriority) then
                -- ...reset the random list
                wipe(randomList);
                randomPriority = priority
              end

              -- Add the mount to the random list if it is high enough priority
              if (priority >= randomPriority) then
                table.insert(randomList, spellId);
              end
            end
          end
        end
         
        -- Check if any suitable mounts were found
        if (next(randomList) ~= nil) then
          -- Choose a random mount
          spellName = randomInfo(randomList[math.random(#(randomList))]);
          break;
        end
      end

      -- Update the button with the mount's spell name
      if (spellName ~= nil) then
        button:SetAttribute("macrotext",
          mountMacroText.."/cast "..spellName);
      else
        button:SetAttribute("macrotext", "");
        UIErrorsFrame:AddMessage(L["No suitable mount"], 1.0, 0.1, 0.1, 1.0);
      end
    end
  end

  return;
end

--
-- Function to initialise info for the mount system
--
function NigglesMountsInitialise()
  -- Work out if the player can have class/race mounts
  if ((Niggles.playerClass == "DRUID") or
      (Niggles.playerClass == "SHAMAN") or
      (Niggles.playerClass == "HUNTER") or
      (Niggles.playerRace  == "Worgen")) then
    hasClassRaceMounts = true;
  else
    hasClassRaceMounts = false;
    NigglesDBC.settings.MountsClassRace = false;
  end

  -- Initialise a list of zones with restricted mounts
  for spellId, zones in pairs(mountRestricted) do
    for zone, _ in pairs(zones) do
      if (zoneRestricted[zone] == nil) then
        zoneRestricted[zone] = {};
      end
      table.insert(zoneRestricted[zone], spellId);
    end
  end

  return;
end

--
-- Function called when the player's spells change
--
function NigglesMountsSpellsUpdate()
  -- Local Variables
  local formName;
  local numContinents = select("#", GetMapContinents());
  local numForms = GetNumShapeshiftForms();
  local spellName;

  -- Work out the player's riding skill
  ridingSkill = 0;
  for idx, spellId in ipairs(ridingInfo) do
    if (IsSpellKnown(spellId)) then
      ridingSkill = idx;
      break;
    end
  end

  -- Clear the continental flight table
  for idx = 1, numContinents do
    continentalFlight[idx] = false;
  end

  -- Check if the player has the skill to fly
  if (ridingSkill >= 3) then
    -- Check if the player can fly in Azeroth
    if (IsSpellKnown(90267)) then -- Flight Pilot's License
      continentalFlight[1] = true;
      continentalFlight[2] = true;
      continentalFlight[5] = true;
    end

    -- Set the flag that indicates the player can fly in Outland
    continentalFlight[3] = true;

    -- Check if the player can fly in Northrend
    if (IsSpellKnown(54197)) then -- Cold Weather Flying
      continentalFlight[4] = true;
    end

    -- Check if the player can fly in Pandaria
    if (IsSpellKnown(115913)) then -- Wisdom of the Four Winds
      continentalFlight[6] = true;
    end
  end

  -- Get names of the mount spells & build a string to cancel their auras
  mountMacroText = "";
  for _, spellInfo in ipairs(mountSpells) do
    spellInfo.name = GetSpellInfo(spellInfo.id);
    if ((spellInfo.name ~= nil) and
        ((spellInfo.flags == 0)) or
         (IsSpellKnown(spellInfo.id))) then
      mountMacroText = mountMacroText.."/cancelaura [combat] "..
        spellInfo.name.."\n";
    end
  end

  return
end

--
-- Function for processing clicks on a Mount toggle button
--
function NigglesMountToggleOnClick(self, button)
  -- Check the toggle is enabled and is being left-clicked.
  if ((self:IsEnabled()) and (button == "LeftButton")) then
    -- Update the state for the mount
    local spellId = self:GetParent().spellID;
    NigglesDBC.mounts[spellId] = bit.bxor((NigglesDBC.mounts[spellId] or 0),
      mountEnv[self:GetID()].flag);
    if (NigglesDBC.mounts[spellId] == 0) then
      NigglesDBC.mounts[spellId] = nil;
    end
    
    -- Filter the mounts
    NigglesMountJournalFilter();
  end

  return;
end

--
-- Function to update the Mount Journal scroll frame
--
function NigglesMountsListUpdate(scrollFrame)
  -- Check if the scroll frame needs updating
  if ((scrollFrame:IsShown()) and 
      (NigglesDBC.settings.MountsEnabled == true) and
      (next(mountJournalToggles))) then
    -- Local Variables
    local numButtons = #scrollFrame.buttons;
    local mountFlags;
    local toggle;
    local button;
    local numEnvironments = #mountEnv;

    -- Update all the buttons in the Mounts scroll frame
    for buttonIdx = 1, numButtons do
      -- Get the current button
      button = scrollFrame.buttons[buttonIdx];

      -- Get the info for the mount the button is for
      if ((button.DragButton:IsEnabled()) and
          (button.index ~= nil)) then
        _, _, _, _, _, mountFlags = GetCompanionInfo("MOUNT", button.index);
      else
        mountFlags = 0;
      end

      -- Set the enabled and checked state of the button's toggles
      for toggleIdx = 1, numEnvironments do
        toggle = mountJournalToggles[buttonIdx][toggleIdx];
        if (bit.band(mountFlags, mountEnv[toggleIdx].flag) ~= 0) then
          toggle:SetEnabled(true);
          if ((NigglesDBC.mounts[button.spellID] ~= nil) and
              (bit.band(NigglesDBC.mounts[button.spellID],
                mountEnv[toggleIdx].flag) ~= 0)) then
            toggle:SetChecked(true);
          else
            toggle:SetChecked(false);
          end
        else
          toggle:SetEnabled(false);
          toggle:SetChecked(false);
        end
      end
    end
  end

  return;
end

--
-- Function to display the tooltip for a Mount toggle button
--
function NigglesMountToggleOnEnter(toggle)
  -- Local Variables
  local toggleId = toggle:GetID();

  -- Lock the highlighting of the toggle's parent
  toggle:GetParent():LockHighlight();

  -- Check the toggle's ID is valid
  if ((toggleId ~= nil) and
      (toggleId >= 1) and
      (toggleId <= #mountEnv)) then
    -- Set the game tooltip
    GameTooltip:SetOwner(toggle, "ANCHOR_RIGHT");
    GameTooltip:SetMinimumWidth(100);
    GameTooltip:SetText(L["Mount"..mountEnv[toggleId].action],
      HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b,
      1, 1);
    GameTooltip:AddLine(L["Mount"..mountEnv[toggleId].action.."Tooltip"],
      NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true);
    GameTooltip:Show();
  else
    -- Hide the game tooltip
    GameTooltip:Hide();
  end

  return;
end

--
-- Function to clear all selected companions/mounts
--
function NigglesOptionsCompanionsClearAll(companionType)
  -- Check the specified type is valid
  if ((companionType == "pets") or (companionType == "mounts")) then
    -- Clear all the selected companions
    wipe(Niggles.updatedOptions[companionType]);

    -- Update any frames that may be affected
    NigglesOptionsRefreshCompanions();
  end

  return;
end

--
-- Function to enable/disable frames in the Niggles Options Companions panel
--
function NigglesOptionsCompanionsEnableFrames()
  -- Local Variables
  local parentName = "NigglesOptionsCompanions";
  local buttonName = parentName.."ClassRaceButton";
  local buttonIdx = 1;
  local addButton;
  local spellIcon;

  -- Enable or disable frames based on the updated options
  NigglesEnableFrames(Niggles.updatedOptions.settings.MountsEnabled,
    NigglesOptionsCompanionsMountsText,
    NigglesOptionsCompanionsMountsHoliday,
    NigglesOptionsCompanionsMountsClear,
    NigglesOptionsCompanionsMountsSelect);
  NigglesEnableFrames(
    ((Niggles.updatedOptions.settings.MountsEnabled) and (hasClassRaceMounts)),
    NigglesOptionsCompanionsMountsClassRace);
  NigglesEnableFrames(Niggles.updatedOptions.settings.PetsEnabled,
    NigglesOptionsCompanionsPetsText,
    NigglesOptionsCompanionsPetsInEKingdoms,
    NigglesOptionsCompanionsPetsInKalimdor,
    NigglesOptionsCompanionsPetsInNorthrend,
    NigglesOptionsCompanionsPetsInOutland,
    NigglesOptionsCompanionsPetsInPandaria,
    NigglesOptionsCompanionsPetsInCities,
    NigglesOptionsCompanionsPetsInDungeons,
    NigglesOptionsCompanionsPetsInRaids,
    NigglesOptionsCompanionsPetsInArenas,
    NigglesOptionsCompanionsPetsInBG,
    NigglesOptionsCompanionsPetsClear,
    NigglesOptionsCompanionsPetsSelect);

  -- Initialise all the class/race buttons to the correct state
  for spellIdx, spellInfo in ipairs(mountSpells) do
    -- Check if a button should be displayed for the spell
    if ((spellInfo.flags ~= 0) and
        ((spellInfo.addButton == nil) or
         (spellInfo.addButton()))) then
      -- Create the button, if required
      button = _G[buttonName..buttonIdx];
      if (button == nil) then
        button = CreateFrame("CHECKBUTTON", buttonName..buttonIdx,
          _G[parentName], "NigglesClassRaceButtonTemplate");
        button:SetPoint("TOPLEFT", _G[buttonName..(buttonIdx-1)],
          "TOPRIGHT", 3, 0);
      end

      -- Check the button exists
      if (button ~= nil) then
        -- Update the button's icon and state
        button.spellId = spellInfo.id
        _, _, spellIcon = GetSpellInfo(spellInfo.id);
        button:SetNormalTexture(spellIcon);
        if ((IsSpellKnown(spellInfo.id)) and
            (Niggles.updatedOptions.settings.MountsEnabled   == true) and
            (Niggles.updatedOptions.settings.MountsClassRace == true)) then
          button:Enable();
          button:SetChecked(Niggles.updatedOptions.mounts[spellInfo.id]);
        else
          button:Disable();
          button:SetChecked(false);
        end
        button.spellFlags = spellInfo.flags;

        -- Increment the button index
        buttonIdx = buttonIdx+1;
      end
    end
  end

  return;
end
  
--
-- Function to refresh the settings for the Interface Options
-- 'Mounts & Companions' panel
--
function NigglesOptionsRefreshCompanions()
  -- Initialise all the widgets to the correct state for the settings
  NigglesOptionsRefreshSettings(NigglesOptionsCompanions);

  -- Initialise all the frames to the correct state for the settings
  NigglesOptionsCompanionsEnableFrames();

  return;
end

--
-- Function to select all companions/mounts as selected
--
function NigglesOptionsCompanionsSelectAll(companionType)
  -- Local Variables
  local spellFlags;
  local spellId;
  local numPets;
  local idealFlags;
  local numCompanions;
  local petTypes = {};
  local petSources = {};
  local petFlags = {};
  local numFlags;

  -- Check the specified type is valid
  if (companionType == "pets") then
    -- Clear all selected pets
    wipe(Niggles.updatedOptions.pets);

    -- Save the current filter settings and then set them to the required state
    for key, value in pairs(petFlagSettings) do
      petFlags[key] = not C_PetJournal.IsFlagFiltered(key);
      C_PetJournal.SetFlagFilter(key, value);
    end
    numFlags = C_PetJournal.GetNumPetTypes();
    for idx = 1, numFlags do
      petTypes[idx] = not C_PetJournal.IsPetTypeFiltered(idx);
    end
    numFlags = C_PetJournal.GetNumPetSources();
    for idx = 1, numFlags do
      petSources[idx] = not C_PetJournal.IsPetSourceFiltered(idx);
    end
    C_PetJournal.AddAllPetTypesFilter();
    C_PetJournal.AddAllPetSourcesFilter();
    C_PetJournal.ClearSearchFilter();

    -- Select all know pets
    numCompanions = C_PetJournal.GetNumPets(false);
    for idx = 1, numCompanions do
      petId, _, _, _, _, _, _, _, _, creatureId =
        C_PetJournal.GetPetInfoByIndex(idx);
      if (petId ~= nil) then
        Niggles.updatedOptions.pets[petId] = creatureId;
      end
    end

    -- Restore the filtering settings
    for key, value in pairs(petFlags) do
      C_PetJournal.SetFlagFilter(key, value);
    end
    for idx, value in ipairs(petTypes) do
      C_PetJournal.SetPetTypeFilter(idx, value);
    end
    for idx, value in ipairs(petSources) do
      C_PetJournal.SetPetSourceFilter(idx, value);
    end
  elseif (companionType == "mounts") then
    -- Clear all selected mounts
    wipe(Niggles.updatedOptions.mounts);

    -- Select all know mounts
    numCompanions = GetNumCompanions("MOUNT");
    for idx = 1, numCompanions do
      _, _, spellId, _, _, spellFlags = GetCompanionInfo("MOUNT", idx);
      if (spellId ~= nil) then
        -- Check if the mount has ideal flags specified for it
        if (mountIdealFlags[spellId] ~= nil) then
          -- Use the ideal flags
          if (mountIdealFlags[spellId] ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = mountIdealFlags[spellId];
          end
        else
          -- Set the mount's flags to select its optimal environment
          if (bit.band(spellFlags, 0x02) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x02;
          elseif (bit.band(spellFlags, 0x01) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x01;
          elseif (bit.band(spellFlags, 0x08) ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = 0x08;
          end
        end
      end
    end
   
    -- Select all known mount spells
    for spellId, spellFlags in pairs(mountSpells) do
      if (IsSpellKnown(spellId)) then
        -- Check if the spell has ideal flags specified for it
        if (mountIdealFlags[spellId] ~= nil) then
          -- Use the spell's ideal flags
          if (mountIdealFlags[spellId] ~= 0) then
            Niggles.updatedOptions.mounts[spellId] = mountIdealFlags[spellId];
          end
        else
          -- Use the spell's standard flags
          Niggles.updatedOptions.mounts[spellId] = spellFlags;
        end
      end
    end
  end

  -- Update any frames that may be affected
  NigglesOptionsRefreshCompanions();

  return;
end

--
-- Function to update the Pet Journal scroll frame
--
function NigglesPetsListUpdate()
  -- Check the Pets option is enabled
  if (NigglesDBC.settings.PetsEnabled == true) then
    -- Local Variables
    local scrollFrame = PetJournal.listScroll;
    local numButtons = #scrollFrame.buttons;
    local toggle;
    local button;

    -- Update all the buttons in the Pets scroll frame
    for idx = 1, numButtons do
      -- Get the current button
      button = scrollFrame.buttons[idx];

      -- Set the enabled and checked state of the button's toggle
      toggle = button.nigglesToggle
      if ((button.dragButton:IsEnabled()) and
          (button.petID ~= nil)) then
        toggle:SetEnabled(true);
        if (NigglesDBC.pets[button.petID] ~= nil) then
          toggle:SetChecked(true);
        else
          toggle:SetChecked(false);
        end
      else
        toggle:SetEnabled(false);
        toggle:SetChecked(false);
      end
    end
  end

  return;
end

--
-- Function for processing clicks on a Pet toggle button
--
function NigglesPetToggleOnClick(self, button)
  -- Local Variables
  local parent = self:GetParent();
  local wasEmpty = (next(NigglesDBC.pets) == nil);

  -- Check the toggle is enabled and is being left-clicked.
  if ((self:IsEnabled()) and
      (button == "LeftButton") and
      (parent ~= nil) and
      (parent.petID ~= nil)) then
    -- Update the state for the pet
    if (self:GetChecked() == 1) then
      NigglesDBC.pets[parent.petID] =
        select(10, C_PetJournal.GetPetInfoByPetID(parent.petID));
    else
      NigglesDBC.pets[parent.petID] = nil;
    end

    -- Register for events, if required
    if (((next(NigglesDBC.pets) ~= nil) and (wasEmpty)) or
        ((next(NigglesDBC.pets) == nil) and (not wasEmpty))) then
      NigglesPetsRegisterEvents();
    end
  end

  return;
end

--
-- Function to display the tooltip for a Pet toggle button
--
function NigglesPetToggleOnEnter(toggle)
  -- Lock the highlighting of the toggle's parent
  toggle:GetParent():LockHighlight();

  -- Set the game tooltip
  GameTooltip:SetOwner(toggle, "ANCHOR_RIGHT");
  GameTooltip:SetMinimumWidth(100);
  GameTooltip:SetText(L["PetTooltip"],
    NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1, 1);
  GameTooltip:Show();

  return;
end

function NigglesMountJournalFilterInitialize(self, level)
  -- Local Variables
  local info = UIDropDownMenu_CreateInfo();
  local numEnvironments = #mountEnv;
  local labelName;

  -- Initialise some variables
  info.keepShownOnClick = true;  

  -- Work out which level of menu needs initialising
  if (level == 1) then
    -- Add the 'Environment' sub-menu
    info.text         = L["Environment"];
    info.value        = 1;
    info.checked      = nil;
    info.isNotRadio   = nil;
    info.func         = nil;
    info.hasArrow     = true;
    info.notCheckable = true;
    UIDropDownMenu_AddButton(info, level)
    
    -- Add the 'Only Selected' menu item, if required
    if (NigglesDBC.settings.MountsEnabled) then
      info.text         = L["Only Selected"];
      info.value        = 0x01;
      info.notCheckable = false;
      info.checked      = mountJournalFilters.onlySelected;
      info.isNotRadio   = true;
      info.hasArrow     = false;
      info.arg1         = "onlySelected";
      info.func         = NigglesMountJournalFilterOnClick;
      UIDropDownMenu_AddButton(info, level);
    end

    -- Add the 'Only Usable' menu item
    info.text         = L["Only Usable"];
    info.value        = 0x01;
    info.notCheckable = false;
    info.checked      = mountJournalFilters.onlyUsable;
    info.isNotRadio   = true;
    info.hasArrow     = false;
    info.arg1         = "onlyUsable";
    info.func         = NigglesMountJournalFilterOnClick;
    UIDropDownMenu_AddButton(info, level);
  elseif (level == 2) then  
    -- Add the 'Check All' menu item
    info.text         = CHECK_ALL
    info.hasArrow     = false;
    info.isNotRadio   = true;
    info.notCheckable = true;
    info.value        = 0xFF;
    info.arg1         = "environment";
    info.func         = NigglesMountJournalFilterOnClick;
    UIDropDownMenu_AddButton(info, level)
    
    -- Add the 'Uncheck All' menu item
    info.text   = UNCHECK_ALL
    info.value  = 0x00;
    UIDropDownMenu_AddButton(info, level)

    info.notCheckable = false;
    for envIdx = 1, numEnvironments do
      info.text    = mountEnv[envIdx].label;
      info.value   = mountEnv[envIdx].flag;
      info.checked = function()
        return (bit.band(mountJournalFilters.environment,
          mountEnv[envIdx].flag) ~= 0);
      end;
      UIDropDownMenu_AddButton(info, level);
    end
  end

  return;
end

--
-- Function to process a button click in the Mount Journal Filter menu
--
function NigglesMountJournalFilterOnClick(self, arg1, arg2, checked)
  -- Check if the menu item is not checkable
  if (self.notCheckable) then
    -- Check/Uncheck all of the menu items in the sub-menu
    mountJournalFilters[arg1] = self.value;
    UIDropDownMenu_Refresh(NigglesMountJournalFilterDropDown, true, 2);
  else
    -- Check if the menu item is boolean
    if (type(mountJournalFilters[arg1]) == "boolean") then
      -- Set the filter's value
      mountJournalFilters[arg1] = checked;
    else
      -- Update the filter's value
      if (checked) then
        mountJournalFilters[arg1] = 
          bit.bor(mountJournalFilters[arg1], self.value);
      else
        mountJournalFilters[arg1] = 
          bit.band(mountJournalFilters[arg1], bit.bnot(self.value));
      end
    end
  end

  -- Filter the mounts
  NigglesMountJournalFilter();

  return;
end
